"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.StationMessageHandler = void 0;
const date_and_time_1 = __importDefault(require("date-and-time"));
const error_1 = require("../error");
const command_1 = require("./command");
const properties_1 = require("./properties");
class StationMessageHandler {
    static async handle(message, driver, client) {
        const { serialNumber, command } = message;
        const station = await driver.getStation(serialNumber);
        switch (command) {
            case command_1.StationCommand.reboot:
                await station.rebootHUB().catch((error) => {
                    throw error;
                });
                return client.schemaVersion >= 13 ? { async: true } : {};
            case command_1.StationCommand.setGuardMode:
                if (client.schemaVersion <= 12) {
                    await station.setGuardMode(message.mode).catch((error) => {
                        throw error;
                    });
                    return {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.isConnected:
                {
                    const result = station.isConnected();
                    if (client.schemaVersion <= 3) {
                        return { connected: result };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: station.getSerial(),
                            connected: result
                        };
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.StationCommand.isConnectedLegacy:
                {
                    if (client.schemaVersion <= 12) {
                        const result = station.isConnected();
                        return { connected: result };
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            /*case StationCommand.getCameraInfo:
                await station.getCameraInfo().catch((error) => {
                    throw error;
                });
                return client.schemaVersion >= 13 ? { async: true } : {};
            case StationCommand.getStorageInfo:
                await station.getStorageInfo().catch((error) => {
                    throw error;
                });
                return client.schemaVersion >= 13 ? { async: true } : {};*/
            case command_1.StationCommand.connect:
                await station.connect().catch((error) => {
                    throw error;
                });
                return {};
            case command_1.StationCommand.disconnect:
                station.close();
                return {};
            case command_1.StationCommand.getPropertiesMetadata:
                {
                    const properties = station.getPropertiesMetadata();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: station.getSerial(),
                            properties: properties
                        };
                    }
                    else {
                        return {
                            serialNumber: station.getSerial(),
                            properties: (0, properties_1.dumpStationPropertiesMetadata)(station, client.schemaVersion)
                        };
                    }
                }
            case command_1.StationCommand.getProperties:
                {
                    const properties = station.getProperties();
                    if (client.schemaVersion <= 3) {
                        return { properties: properties };
                    }
                    else if (client.schemaVersion >= 4) {
                        return {
                            serialNumber: station.getSerial(),
                            properties: properties
                        };
                    }
                    else {
                        return {
                            serialNumber: station.getSerial(),
                            properties: (0, properties_1.dumpStationProperties)(station, client.schemaVersion)
                        };
                    }
                }
            case command_1.StationCommand.setProperty:
                await driver.setStationProperty(serialNumber, message.name, message.value).catch((error) => {
                    throw error;
                });
                return client.schemaVersion >= 13 ? { async: true } : {};
            case command_1.StationCommand.triggerAlarm:
                if (client.schemaVersion >= 3) {
                    await station.triggerStationAlarmSound(message.seconds).catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.resetAlarm:
                if (client.schemaVersion >= 3) {
                    await station.resetStationAlarmSound().catch((error) => {
                        throw error;
                    });
                    return client.schemaVersion >= 13 ? { async: true } : {};
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.hasProperty:
                {
                    if (client.schemaVersion >= 3) {
                        const result = station.hasProperty(message.propertyName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: station.getSerial(),
                                exists: result
                            };
                        }
                        else {
                            throw new error_1.UnknownCommandError(command);
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.StationCommand.hasCommand:
                {
                    if (client.schemaVersion >= 3) {
                        const result = station.hasCommand(message.commandName);
                        if (client.schemaVersion === 3) {
                            return { exists: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: station.getSerial(),
                                exists: result
                            };
                        }
                        else {
                            throw new error_1.UnknownCommandError(command);
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.StationCommand.getCommands:
                {
                    if (client.schemaVersion >= 3) {
                        const result = station.getCommands();
                        if (client.schemaVersion === 3) {
                            return { commands: result };
                        }
                        else if (client.schemaVersion >= 4) {
                            return {
                                serialNumber: station.getSerial(),
                                commands: result
                            };
                        }
                        else {
                            throw new error_1.UnknownCommandError(command);
                        }
                    }
                    else {
                        throw new error_1.UnknownCommandError(command);
                    }
                }
            case command_1.StationCommand.chime:
                if (client.schemaVersion >= 15) {
                    const ringtone = message.ringtone;
                    await station.chimeHomebase(ringtone !== undefined ? ringtone : 0).catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.downloadImage:
                if (client.schemaVersion >= 17) {
                    const file = message.file;
                    await station.downloadImage(file).catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.databaseQueryLatestInfo:
                if (client.schemaVersion >= 18) {
                    await station.databaseQueryLatestInfo().catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.databaseQueryLocal:
                if (client.schemaVersion >= 18) {
                    const serialNumbers = message.serialNumbers;
                    const startDate = date_and_time_1.default.parse(message.startDate, "YYYYMMDD");
                    const endDate = date_and_time_1.default.parse(message.endDate, "YYYYMMDD");
                    const eventType = message.eventType;
                    const detectionType = message.detectionType;
                    const storageType = message.storageType;
                    await station.databaseQueryLocal(serialNumbers, startDate, endDate, eventType, detectionType, storageType).catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.databaseCountByDate:
                if (client.schemaVersion >= 18) {
                    const startDate = date_and_time_1.default.parse(message.startDate, "YYYYMMDD");
                    const endDate = date_and_time_1.default.parse(message.endDate, "YYYYMMDD");
                    await station.databaseCountByDate(startDate, endDate).catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            case command_1.StationCommand.databaseDelete:
                if (client.schemaVersion >= 18) {
                    const ids = message.ids;
                    await station.databaseDelete(ids).catch((error) => {
                        throw error;
                    });
                    return { async: true };
                }
                else {
                    throw new error_1.UnknownCommandError(command);
                }
            default:
                throw new error_1.UnknownCommandError(command);
        }
    }
}
exports.StationMessageHandler = StationMessageHandler;
//# sourceMappingURL=message_handler.js.map